unit CommCtrlEx;

interface

uses Messages, Windows, CommCtrl, Controls;

const
  ICC_LISTVIEW_CLASSES = $00000001; // listview, header
  ICC_TREEVIEW_CLASSES = $00000002; // treeview, tooltips
  ICC_BAR_CLASSES      = $00000004; // toolbar, statusbar, trackbar, tooltips
  ICC_TAB_CLASSES      = $00000008; // tab, tooltips
  ICC_UPDOWN_CLASS     = $00000010; // updown
  ICC_PROGRESS_CLASS   = $00000020; // progress
  ICC_HOTKEY_CLASS     = $00000040; // hotkey
  ICC_ANIMATE_CLASS    = $00000080; // animate
  ICC_WIN95_CLASSES    = $000000FF;
  ICC_DATE_CLASSES     = $00000100; // month picker, date picker, time picker, updown
  ICC_USEREX_CLASSES   = $00000200; // comboex
  ICC_COOL_CLASSES     = $00000400; // rebar (coolbar) control

type
  PInitCommonControlsEx = ^TInitCommonControlsEx;
  TInitCommonControlsEx = Packed Record
    dwSize: Longint;             // size of this structure
    dwICC: Longint;              // flags indicating which classes to be initialized
  end;

  function InitCommonControlsEx(ICCEX: PInitCommonControlsEx): Boolean; StdCall;

const

{ ====== WM_NOTIFY codes (NMHDR.code values) ================== }
  MCN_FIRST                = 0-750;       // monthcal
  MCN_LAST                 = 0-759;

  DTN_FIRST                = 0-760;       // datetimepick
  DTN_LAST                 = 0-769;

  CBEN_FIRST               = 0-800;       // combo box ex
  CBEN_LAST                = 0-830;

  RBN_FIRST                = 0-831;       // rebar
  RBN_LAST                 = 0-859;

{ ==================== CUSTOM DRAW ========================================== }

  // custom draw return flags
  // values under 0x00010000 are reserved for global custom draw values.
  // above that are for specific controls
  CDRF_DODEFAULT          = $00000000;
  CDRF_NEWFONT            = $00000002;
  CDRF_SKIPDEFAULT        = $00000004;

  CDRF_NOTIFYPOSTPAINT    = $00000010;
  CDRF_NOTIFYITEMDRAW     = $00000020;

  // drawstage flags
  // values under 0x00010000 are reserved for global custom draw values.
  // above that are for specific controls
  CDDS_PREPAINT           = $000000001;
  CDDS_POSTPAINT          = $000000002;
  // the 0x000010000 bit means it's individual item specific
  CDDS_ITEM               = $000010000;
  CDDS_ITEMPREPAINT       = (CDDS_ITEM or CDDS_PREPAINT);
  CDDS_ITEMPOSTPAINT      = (CDDS_ITEM or CDDS_POSTPAINT);

  // itemState flags
  CDIS_SELECTED           = $0001;
  CDIS_GRAYED             = $0002;
  CDIS_DISABLED           = $0004;
  CDIS_CHECKED            = $0008;
  CDIS_FOCUS              = $0010;
  CDIS_DEFAULT            = $0020;

type
  PNMCustomDraw = ^TNMCustomDraw;
  TNMCustomDraw = Packed Record
    Hdr: TNMHDR;
    dwDrawStage: Longint;
    hDC: HDC;
    rc: TRect;
    dwItemSpec: Longint;  // this is control specific, but it's how to specify an item.  valid only with CDDS_ITEM bit set
    uItemState: UINT;
    lItemlParam: LPARAM;
  end;

  // for tooltips
  PNMTTCustomDraw = ^TNMTTCustomDraw;
  TNMTTCustomDraw = Packed Record
    nmcd: TNMCustomDraw;
    uDrawFlags: UINT;
  end;

//====== LISTVIEW CONTROL =====================================================
const
  LVS_OWNERDATA  = $1000;

  LVIF_INDENT      = $0010;
  LVIF_NORECOMPUTE = $0800;

  LVCF_IMAGE              = $0010;
  LVCF_ORDER              = $0020;
  LVCFMT_IMAGE            = $0800;
  LVCFMT_BITMAP_ON_RIGHT  = $1000;
  LVCFMT_COL_HAS_IMAGES   = $8000;

const
  LVM_SETICONSPACING      = (LVM_FIRST + 53);
  // -1 for cx and cy means we'll use the default (system settings)
  // 0 for cx or cy means use the current setting (allows you to change just one param)
  function ListView_SetIconSpacing(hwndLV: HWND; cx, cy: integer): DWORD; stdcall;
      //  (DWORD)SNDMSG((hwndLV), LVM_SETICONSPACING, 0, MAKELONG(cx,cy))

const
  LVM_SETEXTENDEDLISTVIEWSTYLE = (LVM_FIRST + 54);
  function ListView_SetExtendedListViewStyle(hwndLV: HWND; dw: DWORD): DWORD; stdcall;
      //  (DWORD)SNDMSG((hwndLV), LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dw)

const
  LVM_GETEXTENDEDLISTVIEWSTYLE = (LVM_FIRST + 55);
  function ListView_GetExtendedListViewStyle(hwndLV: HWND): DWORD; stdcall;
     //   (DWORD)SNDMSG((hwndLV), LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0)

const
  LVS_EX_GRIDLINES        = $000000001;
  LVS_EX_SUBITEMIMAGES    = $000000002;
  LVS_EX_CHECKBOXES       = $000000004;
  LVS_EX_TRACKSELECT      = $000000008;
  LVS_EX_HEADERDRAGDROP   = $000000010;
  LVS_EX_FULLROWSELECT    = $000000020; // applies to report mode only
  LVS_EX_ONECLICKACTIVATE = $000000040;
  LVS_EX_TWOCLICKACTIVATE = $000000080;

const
  LVM_GETSUBITEMRECT      = (LVM_FIRST + 56);
  function ListView_GetSubItemRect(hwnd: HWND; iItem, iSubItem, code: integer; var prc: TRect): Bool; stdcall;
   //     (BOOL)SNDMSG((hwnd), LVM_GETSUBITEMRECT, (WPARAM)(int)(iItem), \
   //             ((prc) ? ((((LPRECT)(prc))->top = iSubItem), (((LPRECT)(prc))->left = code), (prc)) : (LPARAM)(LPRECT)NULL))

const
  LVM_SUBITEMHITTEST      = (LVM_FIRST + 57);
  function ListView_SubItemHitTest(hwnd: HWND; var plvhti: TLVHitTestInfo): integer; stdcall;
    //    (int)SNDMSG((hwnd), LVM_SUBITEMHITTEST, 0, (LPARAM)(LPLVHITTESTINFO)(plvhti))

const
  LVM_SETCOLUMNORDERARRAY = (LVM_FIRST + 58);
  function ListView_SetColumnOrderArray(hwnd: HWND; iCount: integer; var pi: integer): Bool; stdcall;
    //    (BOOL)SNDMSG((hwnd), LVM_SETCOLUMNORDERARRAY, (WPARAM)iCount, (LPARAM)(LPINT)pi)

const
  LVM_GETCOLUMNORDERARRAY = (LVM_FIRST + 59);
  function ListView_GetColumnOrderArray(hwnd: HWND; iCount: integer; var pi: integer): Bool; stdcall;
    //    (BOOL)SNDMSG((hwnd), LVM_GETCOLUMNORDERARRAY, (WPARAM)iCount, (LPARAM)(LPINT)pi)

const
  LVM_SETHOTITEM = (LVM_FIRST + 60);
  function ListView_SetHotItem(hwnd: HWND; i: integer): integer; stdcall;
    //    (int)SNDMSG((hwnd), LVM_SETHOTITEM, (WPARAM)i, 0)

const
  LVM_GETHOTITEM = (LVM_FIRST + 61);
  function ListView_GetHotItem(hwnd: HWND): integer; stdcall;
    //    (int)SNDMSG((hwnd), LVM_GETHOTITEM, 0, 0)

const
  LVM_SETHOTCURSOR = (LVM_FIRST + 62);
  function ListView_SetHotFont(hwnd: HWND; hcur: TCursor): TCursor; stdcall;
    //    (HCURSOR)SNDMSG((hwnd), LVM_SETHOTCURSOR, (WPARAM)hcur, 0)

const
  LVM_GETHOTCURSOR = (LVM_FIRST + 63);
  function ListView_GetHotFont(hwnd: HWND): TCursor; stdcall;
   //     (HCURSOR)SNDMSG((hwnd), LVM_GETHOTCURSOR, 0, 0)

const
  LVM_APPROXIMATEVIEWRECT = (LVM_FIRST + 64);
  function ListView_ApproximateViewRect(hwnd: HWND; iWidth, iHeight, iCount: integer): DWORD; stdcall;
   //   (DWORD)SNDMSG((hwnd), LVM_APPROXIMATEVIEWRECT, iCount, MAKELPARAM(iWidth, iHeight))

type
  PNMLVCustomDraw = ^TNMLVCustomDraw;
  TNMLVCustomDraw = packed record
    nmcd: TNMCustomDraw;
    clrText: COLORREF;
    clrTextBk: COLORREF;
  end;

  PNMLVCacheHint = ^TNMLVCacheHint;
  TNMLVCacheHint = packed record
    hdr: TNMHDR;
    iFrom: integer;
    iTo: integer;
   end;

   LPNM_CACHEHINT = PNMLVCacheHint;
   PNM_CACHEHINT  = PNMLVCacheHint;
   NM_CACHEHINT   = TNMLVCacheHint;

   PNMLVFindItem = ^TNMLVFindItem;
   TNMLVFindItem = packed record
      hdr: TNMHDR;
      iStart: integer;
      lvfi: TLVFindInfo;
   end;

   PNM_FINDITEM   = PNMLVFindItem;
   LPNM_FINDITEM  = PNMLVFindItem;
   NM_FINDITEM    = TNMLVFindItem;

const
   LVN_ODCACHEHINT = (LVN_FIRST-13);
   LVN_ODFINDITEMA = (LVN_FIRST-52);
   LVN_ODFINDITEMW = (LVN_FIRST-79);

   LVN_ITEMACTIVATE = (LVN_FIRST-14);

   LVN_ODFINDITEM = LVN_ODFINDITEMA;

   LVN_MARQUEEBEGIN = (LVN_FIRST-56);

//====== TREEVIEW CONTROL =====================================================
const
  TVS_SHAREDIMAGELISTS   = $0000;
  TVS_PRIVATEIMAGELISTS  = $0040;

  TVS_NOTOOLTIPS         = $0080;
  TVS_CHECKBOXES         = $0100;
  TVS_TRACKSELECT        = $0200;

  TVIS_EXPANDPARTIAL     = $0080;
  TVE_EXPANDPARTIAL      = $4000;

type
  PNMTVCustomDraw = ^TNMTVCustomDraw;
  TNMTVCustomDraw = packed record
    nmcd: TNMCustomDraw;
    clrText: COLORREF;
    clrTextBk: COLORREF;
  end;

//====== IMAGE APIS ===========================================================

const
  ILD_BLEND  =  ILD_BLEND50;


//====== REBAR CONTROL ========================================================

const
  REBARCLASSNAME  = 'ReBarWindow';

  RBIM_STYLE      = $00000001;
  RBIM_IMAGELIST  = $00000002;
  RBIM_BACKGROUND = $00000004;

  RBS_TOOLTIPS    = $00000001;
  RBS_VARHEIGHT   = $00000002;
  RBS_VALID       = (RBS_TOOLTIPS or RBS_VARHEIGHT);

type
  PRebarInfo = ^TRebarInfo;
  TRebarInfo = packed record
    cbSize: UINT;
    fMask: UINT;
    fStyle: UINT;
    himl: HImageList;
    hbmBack: HBITMAP;
  end;

const
  RBBS_BREAK      = $00000001;  // break to new line
  RBBS_FIXEDSIZE  = $00000002;  // band can't be sized
  RBBS_KEEPHEIGHT = $00000004;  // don't change height
  RBBS_HIDDEN     = $00000008;  // don't show
  RBBS_NOVERT     = $00000010;  // don't show when vertical

  RBBIM_STYLE     = $00000001;
  RBBIM_COLORS    = $00000002;
  RBBIM_TEXT      = $00000004;
  RBBIM_IMAGE     = $00000008;
  RBBIM_CHILD     = $00000010;
  RBBIM_CHILDSIZE = $00000020;
  RBBIM_SIZE      = $00000040;

type
  PRebarBandInfoA =^TRebarBandInfoA;
  TRebarBandInfoA = packed record
    cbSize: UINT;
    fMask: UINT;
    fStyle: UINT;
    clrFore: COLORREF;
    clrBack: COLORREF;
    lpText: PAnsiChar;
    cch: UINT;
    iImage: integer;
    hwndChild: HWND;
    cxMinChild: UINT;
    cyMinChild: UINT;
    cx: UINT;
    hbmBack: HBITMAP;
    wID: UINT;
  end;

  PRebarBandInfoW =^TRebarBandInfoW;
  TRebarBandInfoW = packed record
    cbSize: UINT;
    fMask: UINT;
    fStyle: UINT;
    clrFore: COLORREF;
    clrBack: COLORREF;
    lpText: PWideChar;
    cch: UINT;
    iImage: integer;
    hwndChild: HWND;
    cxMinChild: UINT;
    cyMinChild: UINT;
    cx: UINT;
    hbmBack: HBITMAP;
    wID: UINT;
  end;

  TRebarBandInfo = TRebarBandInfoA;

const
  RB_INSERTBANDA  = (WM_USER +  1);
  RB_DELETEBAND   = (WM_USER +  2);
  RB_GETBARINFO   = (WM_USER +  3);
  RB_SETBARINFO   = (WM_USER +  4);
  RB_GETBANDINFO  = (WM_USER +  5);
  RB_SETBANDINFOA = (WM_USER +  6);
  RB_SETPARENT    = (WM_USER +  7);
  RB_ERASEDARK    = (WM_USER +  8);
  RB_ANIMATE      = (WM_USER +  9);
  RB_INSERTBANDW  = (WM_USER +  10);
  RB_SETBANDINFOW = (WM_USER +  11);
  RB_GETBANDCOUNT = (WM_USER +  12);
  RB_GETROWCOUNT  = (WM_USER +  13);
  RB_GETROWHEIGHT = (WM_USER +  14);

  RB_INSERTBAND   = RB_INSERTBANDA;
  RB_SETBANDINFO  = RB_SETBANDINFOA;

  RBN_HEIGHTCHANGE = (RBN_FIRST - 0);

//====== MONTHCAL CONTROL ======================================================
const
  MONTHCAL_CLASS    = 'SysMonthCal32';

type
  MonthDayState = DWORD;

  TNMDayState = packed record
    nmhdr: TNMHDR;  // this must be first, so we don't break WM_NOTIFY
    stStart: TSYSTEMTIME;
    cDayState: integer;
    rgDayState: array[0..1] of MonthDayState;
  end;

  TNMSelChange = packed record
    nmhdr: TNMHDR;  // this must be first, so we don't break WM_NOTIFY
    stSelStart: TSYSTEMTIME;
    stSelEnd: TSYSTEMTIME;
  end;

const

  MCM_FIRST = $1000;

  // BOOL MonthCal_GetCurSel(HWND hmc, LPSYSTEMTIME pst)
  //   returns FALSE if MCS_MULTISELECT
  //   returns TRUE and sets *pst to the currently selected date otherwise
const
  MCM_GETCURSEL = (MCM_FIRST + 1);
  function MonthCal_GetCurSel(hmc: HWND; const pst: TSYSTEMTIME): Bool; stdcall;
     //SNDMSG(hmc, MCM_GETCURSEL, 0, (LPARAM)(pst))

  // BOOL MonthCal_SetCurSel(HWND hmc, LPSYSTEMTIME pst)
  //   returns FALSE if MCS_MULTISELECT
  //   returns TURE and sets the currently selected date to *pst otherwise
const
  MCM_SETCURSEL = (MCM_FIRST + 2);
  function MonthCal_SetCurSel(hmc: HWND; const pst: TSYSTEMTIME): Bool; stdcall;
    //SNDMSG(hmc, MCM_SETCURSEL, 0, (LPARAM)(pst))

  // DWORD MonthCal_GetMaxSelCount(HWND hmc)
  //   returns the maximum number of selectable days allowed
const
  MCM_GETMAXSELCOUNT = (MCM_FIRST + 3);
  function MonthCal_GetMaxSelCount(hmc: HWND): DWORD; stdcall;
   //SNDMSG(hmc, MCM_GETMAXSELCOUNT, 0, 0L)

  // BOOL MonthCal_SetMaxSelCount(HWND hmc, UINT n)
  //   sets the max number days that can be selected iff MCS_MULTISELECT
const
  MCM_SETMAXSELCOUNT = (MCM_FIRST + 4);
  function MonthCal_SetMaxSelCount(hmc: HWND; n: Longint): Bool; stdcall;
    //SNDMSG(hmc, MCM_SETMAXSELCOUNT, (WPARAM)(n), 0L)

  // BOOL MonthCal_GetSelRange(HWND hmc, LPSYSTEMTIME rgst)
  //   sets rgst[0] to the first day of the selection range
  //   sets rgst[1] to the last day of the selection range
const
  MCM_GETSELRANGE = (MCM_FIRST + 5);
  function MonthCal_GetSelRange(hmc: HWND; const rgst: array of TSYSTEMTIME): Bool; stdcall;
    //SNDMSG(hmc, MCM_GETSELRANGE, 0, (LPRARAM)(rgst))

  // BOOL MonthCal_SetSelRange(HWND hmc, LPSYSTEMTIME rgst)
  //   selects the range of days from rgst[0] to rgst[1]
const
  MCM_SETSELRANGE  = (MCM_FIRST + 6);
  function MonthCal_SetSelRange(hmc: HWND; const rgst: array of TSYSTEMTIME): Bool; stdcall;
    //SNDMSG(hmc, MCM_SETSELRANGE, 0, (LPARAM)(rgst))

  // DWORD MonthCal_GetMonthRange(HWND hmc, DWORD gmr, LPSYSTEMTIME rgst)
  //   if rgst specified, sets rgst[0] to the starting date and
  //      and rgst[1] to the ending date of the the selectable (non-grayed)
  //      days if GMR_VISIBLE or all the displayed days (including grayed)
  //      if GMR_DAYSTATE.
  //   returns the number of months spanned by the above range.
const
  MCM_GETMONTHRANGE = (MCM_FIRST + 7);
  function MonthCal_GetMonthRange(hmc: HWND; gmr: DWORD; const rgst: array of TSYSTEMTIME): DWORD; stdcall;
    //SNDMSG(hmc, MCM_GETMONTHRANGE, (WPARAM)(gmr), (LPARAM)(rgst))

  // BOOL MonthCal_SetDayState(HWND hmc, int cbds, DAYSTATE *rgds)
  //   cbds is the count of DAYSTATE items in rgds and it must be equal
  //   to the value returned from MonthCal_GetMonthRange(hmc, GMR_DAYSTATE, NULL)
  //   This sets the DAYSTATE bits for each month (grayed and non-grayed
  //   days) displayed in the calendar. The first bit in a month's DAYSTATE
  //   corresponts to bolding day 1, the second bit affects day 2, etc.
const
  MCM_SETDAYSTATE = (MCM_FIRST + 8);
  function MonthCal_SetDayState(hmc: HWND; cbds: integer; const rgds: array of MonthDayState): Bool; stdcall;
    // SNDMSG(hmc, MCM_SETDAYSTATE, (WPARAM)(cbds), (LPARAM)(rgds))

  // BOOL MonthCal_GetMinReqRect(HWND hmc, LPRECT prc)
  //   sets *prc the minimal size needed to display one month
const
  MCM_GETMINREQRECT = (MCM_FIRST + 9);
  function MonthCal_GetMinReqRect(hmc: HWND; const prc: TRect): Bool; stdcall;
    // SNDMSG(hmc, MCM_GETMINREQRECT, 0, (LPARAM)(prc))

type
  PMCHitTestInfo = ^TMCHitTestInfo;
  TMCHitTestInfo = packed record
    cbSize: UINT;
    pt: TPoint;
    uHit: UINT;   // out param
    st: TSYSTEMTIME;
  end;

const
  MCM_HITTEST = (MCM_FIRST + 14);
  procedure MonthCal_HitTest(hmc: HWND; var pinfo: TMCHitTestInfo); stdcall;
     //SNDMSG(hmc, MCM_HITTEST, 0, (LPARAM)(PMCHITTESTINFO)pinfo)

const
  MCHT_TITLE        = $00010000;
  MCHT_CALENDAR     = $00020000;
  MCHT_TODAYLINK    = $00030000;

  MCHT_NEXT         = $01000000;   // these indicate that hitting
  MCHT_PREV         = $02000000;   // here will go to the next/prev month

  MCHT_NOWHERE      = $00000000;

  MCHT_TITLEBK      = (MCHT_TITLE);
  MCHT_TITLEMONTH   = (MCHT_TITLE or $0001);
  MCHT_TITLEYEAR    = (MCHT_TITLE or $0002);
  MCHT_TITLEBTNNEXT = (MCHT_TITLE or MCHT_NEXT or $0003);
  MCHT_TITLEBTNPREV = (MCHT_TITLE or MCHT_PREV or $0003);

  MCHT_CALENDARBK   = (MCHT_CALENDAR);
  MCHT_CALENDARDATE = (MCHT_CALENDAR or $0001);
  MCHT_CALENDARDATENEXT = (MCHT_CALENDARDATE or MCHT_NEXT);
  MCHT_CALENDARDATEPREV = (MCHT_CALENDARDATE or MCHT_PREV);
  MCHT_CALENDARDAY      = (MCHT_CALENDAR or $0002);
  MCHT_CALENDARWEEKNUM  = (MCHT_CALENDAR or $0003);

  MCM_SETCOLOR      = (MCM_FIRST + 10);
  procedure MonthCal_SetColor(hmc: HWND; iColor: DWORD; clr: DWORD); stdcall;
    //SNDMSG(hmc, MCM_SETCOLOR, iColor, clr)

const
  MCM_GETCOLOR      = (MCM_FIRST + 11);
  procedure MonthCal_GetColor(hmc: HWND; iColor: DWORD); stdcall;
    // SNDMSG(hmc, MCM_SETCOLOR, iColor, 0)

const
  MCSC_BACKGROUND   = 0;   // the background color (between months)
  MCSC_TEXT         = 1;   // the dates
  MCSC_TITLEBK      = 2;   // background of the title
  MCSC_TITLETEXT    = 3;
  MCSC_MONTHBK      = 4;   // background within the month cal
  MCSC_TRAILINGTEXT = 5;   // the text color of header & trailing days

  MCN_SELCHANGE     = (MCN_FIRST + 1);
  MCN_GETDAYSTATE   = (MCN_FIRST + 2);

  MCS_DAYSTATE      =  $0001;
  MCS_MULTISELECT   =  $0002;
  MCS_WEEKNUMBERS   =  $0004;
  MCS_NOTODAY       =  $0008;

  GMR_VISIBLE       =  0;       // visible portion of display
  GMR_DAYSTATE      =  1;       // above plus the grayed out parts of
                                // partially displayed months


//====== DATETIMEPICK CONTROL ==================================================
const
  DATETIMEPICK_CLASS    =  'SysDateTimePick32';

  DTM_FIRST             =   $1000;

  // DWORD DateTimePick_GetSystemtime(HWND hdp, LPSYSTEMTIME pst)
  //   returns GDT_NONE if "none" is selected (DTS_SHOWNONE only)
  //   returns GDT_VALID and modifies *pst to be the currently selected value
  DTM_GETSYSTEMTIME     =   (DTM_FIRST + 1);

function DateTime_GetSystemtime(hdp: HWND; const pst: TSYSTEMTIME): DWORD; stdcall;

  // BOOL DateTime_SetSystemtime(HWND hdp, DWORD gd, LPSYSTEMTIME pst)
  //   if gd==GDT_NONE, sets datetimepick to None (DTS_SHOWNONE only)
  //   if gd==GDT_VALID, sets datetimepick to *pst
  //   returns TRUE on success, FALSE on error (such as bad params)
const
  DTM_SETSYSTEMTIME     =    (DTM_FIRST + 2);

function DateTime_SetSystemtime(hdp: HWND; gd: DWORD; const pst: TSYSTEMTIME): Bool; stdcall;

  // DWORD DateTime_GetRange(HWND hdp, LPSYSTEMTIME rgst)
  //   modifies rgst[0] to be the minimum ALLOWABLE systemtime (or 0 if no minimum)
  //   modifies rgst[1] to be the maximum ALLOWABLE systemtime (or 0 if no maximum)
  //   returns GDTR_MIN|GDTR_MAX if there is a minimum|maximum limit
const
  DTM_GETRANGE         =     (DTM_FIRST + 3);

function DateTime_GetRange(hdp: HWND; const rgst: array of TSYSTEMTIME): DWORD; stdcall;

  // BOOL DateTime_SetRange(HWND hdp, DWORD gdtr, LPSYSTEMTIME rgst)
  //   if GDTR_MIN, sets the minimum ALLOWABLE systemtime to rgst[0], otherwise removes minimum
  //   if GDTR_MAX, sets the maximum ALLOWABLE systemtime to rgst[1], otherwise removes maximum
  //   returns TRUE on success, FALSE on error (such as invalid parameters)
const
  DTM_SETRANGE         =      (DTM_FIRST + 4);

function DateTime_SetRange(hdp: HWND; gd: DWORD; const rgst: array of TSYSTEMTIME): Bool; stdcall;

  // BOOL DateTime_SetFormat(HWND hdp, LPCTSTR sz)
  //   sets the display formatting string to sz (see GetDateFormat and GetTimeFormat for valid formatting chars)
  //   NOTE: 'X' is a valid formatting character which indicates that the application
  //   will determine how to display information. Such apps must support DTN_WMKEYDOWN,
  //   DTN_FORMAT, and DTN_FORMATQUERY.
const
  DTM_SETFORMAT        =      (DTM_FIRST + 5);

function DateTime_SetFormat(hdp: HWND; sz: PChar): Bool; stdcall;

const
  DTM_SETMCCOLOR       =      (DTM_FIRST + 6);

function DateTime_SetMonthCalColor(hdp: HWND; iColor: DWORD; clr: DWORD): DWORD; stdcall;

const
  DTM_GETMCCOLOR       =      (DTM_FIRST + 7);

function DateTime_GetMonthCalColor(hdp: HWND; iColor: DWORD): DWORD; stdcall;

const
  DTM_GETMONTHCAL      =      (DTM_FIRST + 8);

function DateTime_GetMonthCal(hdp: HWND): HWND;

const
  DTS_UPDOWN           = $0001; // use UPDOWN instead of MONTHCAL
  DTS_SHOWNONE         = $0002; // allow a NONE selection
  DTS_SHORTDATEFORMAT  = $0000; // use the short date format (app must forward WM_WININICHANGE messages)
  DTS_LONGDATEFORMAT   = $0004; // use the long date format (app must forward WM_WININICHANGE messages)
  DTS_TIMEFORMAT       = $0009; // use the time format (app must forward WM_WININICHANGE messages)
  DTS_APPCANPARSE      = $0010; // allow user entered strings (app MUST respond to DTN_USERSTRING)

  DTN_DATETIMECHANGE   = (DTN_FIRST + 1); // the systemtime has changed

type
  PNMDateTimeChange = ^TNMDateTimeChange;
  TNMDateTimeChange = Packed Record
    nmhdr: TNMHDR;
    dwFlags: Longint;    // GDT_VALID or GDT_NONE
    st: TSYSTEMTIME;         // valid iff dwFlags==GDT_VALID
  end;

const
  DTN_USERSTRING       = (DTN_FIRST + 2); // the user has entered a string

type
  PNMDateTimeString = ^TNMDateTimeString;
  TNMDateTimeString = Packed Record
    nmhdr: TNMHDR;
    pszUserString: PChar;  // string user entered
    st: TSYSTEMTIME;        // app fills this in
    dwFlags: DWORD;        // GDT_VALID or GDT_NONE
  end;

const
  DTN_WMKEYDOWN        = (DTN_FIRST + 3); // modify keydown on app format field (X)

type
  PNMDateTimeWMKeyDown = ^TNMDateTimeWMKeyDown;
  TNMDateTimeWMKeyDown = Packed Record
    nmhdr:   TNMHDR;
    nVirtKey: integer;  // virtual key code of WM_KEYDOWN which MODIFIES an X field
    pszFormat: PChar;   // format substring
    st: TSYSTEMTIME;     // current systemtime, app should modify based on key
  end;

const
  DTN_FORMAT        = (DTN_FIRST + 4); // query display for app format field (X)

type
  PNMDateTimeFormat = ^TNMDateTimeFormat;
  TNMDateTimeFormat = packed record
    nmhdr: TNMHDR;
    pszFormat: PChar;   // format substring
    st: TSYSTEMTIME;       // current systemtime
    pszDisplay: PChar;  // string to display
    szDisplay: array [0..64] of Char; // buffer pszDisplay originally points at
  end;

const
  DTN_FORMATQUERY  = (DTN_FIRST + 5); // query formatting info for app format field (X)
type
  PNMDateTimeFormationQuery = ^TNMDateTimeFormationQuery;
  TNMDateTimeFormationQuery = packed record
    nmhdr: TNMHDR;
    pszFormat: PChar; // format substring
    szMax: integer;   // max bounding rectangle app will use for this format string
  end;

const
  DTN_DROPDOWN  = (DTN_FIRST + 6); //not yet correctly
  DTN_ClOSEUP   = (DTN_FIRST + 7); //not yet correctly

const
  GDTR_MIN     = $0001;
  GDTR_MAX     = $0002;

  GDT_ERROR    = -1;
  GDT_VALID    =  0;
  GDT_NONE     =  1;

function ConvertSystemTimeToDataTime(SystemTime: TSYSTEMTIME): TDateTime;
function ConvertDateTimeToSystemTime(DateTime: TDateTime): TSYSTEMTIME;


implementation

uses SysUtils;

const
  cctrl = 'comctl32.dll';

function InitCommonControlsEx(ICCEX: PInitCommonControlsEx): Boolean; external cctrl name 'InitCommonControlsEx';

function ConvertSystemTimeToDataTime(SystemTime: TSYSTEMTIME): TDateTime;
var
  DateTimeStr: string;
begin
  with SystemTime do begin
    Result := EncodeDate(wYear, wMonth, wDay);
    Result := Result + EncodeTime(wHour, wMinute, wSecond, wMilliseconds);
  end;
end;

function ConvertDateTimeToSystemTime(DateTime: TDateTime): TSYSTEMTIME;
var
  Year, Month, Day, Hour, Min, Sec, MSec: Word;
begin
  DecodeDate(DateTime, Year, Month, Day);
  DecodeTime(DateTime, Hour, Min, Sec, MSec);
  with Result do begin
    wMonth := Month;
    wDay := Day;
    wYear := Year;
    wHour := Hour;
    wMinute := Min;
    wSecond := Sec;
  end;
end;

function DateTime_GetSystemtime(hdp: HWND; const pst: TSYSTEMTIME): DWORD;
begin
  result := DWORD( SendMessage(hdp, DTM_GETSYSTEMTIME, 0, Longint(@pst)) );
end;

function DateTime_SetSystemtime(hdp: HWND; gd: DWORD; const pst: TSYSTEMTIME): Bool;
begin
 result := BOOL( SendMessage(hdp, DTM_SETSYSTEMTIME, Longint(gd), Longint(@pst)) );
end;

function DateTime_GetRange(hdp: HWND; const rgst: array of TSYSTEMTIME): DWORD;
begin
  result := DWORD( SendMessage(hdp, DTM_GETRANGE, 0, longint(@rgst)) );
end;

function DateTime_SetRange(hdp: HWND; gd: DWORD; const rgst: array of TSYSTEMTIME): Bool;
begin
  result := Bool( SendMessage(hdp, DTM_SETRANGE, Longint(gd), Longint(@rgst)) );
end;

function DateTime_SetFormat(hdp: HWND; sz: PChar): Bool;
begin
  result := Bool( SendMessage(hdp, DTM_SETFORMAT, 0, Longint(sz)) );
end;

function DateTime_SetMonthCalColor(hdp: HWND; iColor: DWORD; clr: DWORD): DWORD;
begin
  result := DWORD( SendMessage(hdp, DTM_SETMCCOLOR, iColor, clr));
end;

function DateTime_GetMonthCalColor(hdp: HWND; iColor: DWORD): DWORD;
begin
  result := DWORD( SendMessage(hdp, DTM_GETMCCOLOR, iColor, 0) );
end;

function DateTime_GetMonthCal(hdp: HWND): HWND;
begin
  result := HWND( SendMessage(hdp, DTM_GETMONTHCAL, 0, 0) );
end;

//====== MONTHCAL CONTROL ======================================================

function MonthCal_GetCurSel(hmc: HWND; const pst: TSYSTEMTIME): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_GETCURSEL, 0, Longint(@pst)) );
end;

function MonthCal_SetCurSel(hmc: HWND; const pst: TSYSTEMTIME): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_SETCURSEL, 0, Longint(@pst)) );
end;

function MonthCal_GetMaxSelCount(hmc: HWND): DWORD;
begin
  result := DWORD( SendMessage(hmc, MCM_GETMAXSELCOUNT, 0, 0) );
end;

function MonthCal_SetMaxSelCount(hmc: HWND; n: Longint): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_SETMAXSELCOUNT, Longint(n), 0) );
end;

function MonthCal_GetSelRange(hmc: HWND; const rgst: array of TSYSTEMTIME): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_GETSELRANGE, 0, Longint(@rgst)) );
end;

function MonthCal_SetSelRange(hmc: HWND; const rgst: array of TSYSTEMTIME): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_SETSELRANGE, 0, Longint(@rgst)) );
end;

function MonthCal_GetMonthRange(hmc: HWND; gmr: DWORD; const rgst: array of TSYSTEMTIME): DWORD;
begin
  result := DWORD( SendMessage(hmc, MCM_GETMONTHRANGE, Longint(gmr), Longint(@rgst)) );
end;

function MonthCal_SetDayState(hmc: HWND; cbds: integer; const rgds: array of MonthDayState): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_SETDAYSTATE, Longint(cbds), Longint(@rgds)) );
end;

function MonthCal_GetMinReqRect(hmc: HWND; const prc: TRect): Bool;
begin
  result := Bool( SendMessage(hmc, MCM_GETMINREQRECT, 0, Longint(@prc)) );
end;

procedure MonthCal_HitTest(hmc: HWND; var pinfo: TMCHitTestInfo);
begin
  SendMessage(hmc, MCM_HITTEST, 0, Longint(@pinfo));
end;

procedure MonthCal_SetColor(hmc: HWND; iColor: DWORD; clr: DWORD);
begin
  SendMessage(hmc, MCM_SETCOLOR, iColor, clr);
end;

procedure MonthCal_GetColor(hmc: HWND; iColor: DWORD);
begin
  SendMessage(hmc, MCM_SETCOLOR, iColor, 0);
end;

//====== LISTVIEW CONTROL =====================================================
// -1 for cx and cy means we'll use the default (system settings)
// 0 for cx or cy means use the current setting (allows you to change just one param)
function ListView_SetIconSpacing(hwndLV: HWND; cx, cy: integer): DWORD;
begin
  result := DWORD( SendMessage(hwndLV, LVM_SETICONSPACING, 0, MAKELONG(cx,cy) ));
end;

function ListView_SetExtendedListViewStyle(hwndLV: HWND; dw: DWORD): DWORD;
begin
  result := DWORD( SendMessage(hwndLV, LVM_SETEXTENDEDLISTVIEWSTYLE, 0, dw) );
end;

function ListView_GetExtendedListViewStyle(hwndLV: HWND): DWORD;
begin
  result := DWORD( SendMessage(hwndLV, LVM_GETEXTENDEDLISTVIEWSTYLE, 0, 0) );
end;

function ListView_GetSubItemRect(hwnd: HWND; iItem, iSubItem, code: integer; var prc: TRect): Bool;
begin
{  result := BOOL( SendMessage(hwnd, LVM_GETSUBITEMRECT, Longint(iItem),
             ((prc) ? ((((LPRECT)(prc))->top = iSubItem), (((LPRECT)(prc))->left = code), (prc)) : (LPARAM)(LPRECT)NULL))}
end;

function ListView_SubItemHitTest(hwnd: HWND; var plvhti: TLVHitTestInfo): integer;
begin
  result := integer( SendMessage(hwnd, LVM_SUBITEMHITTEST, 0, Longint(@plvhti)) );
end;

function ListView_SetColumnOrderArray(hwnd: HWND; iCount: integer; var pi: integer): Bool;
begin
  result := BOOL( SendMessage(hwnd, LVM_SETCOLUMNORDERARRAY, Longint(iCount), Longint(pi)) );
end;

function ListView_GetColumnOrderArray(hwnd: HWND; iCount: integer; var pi: integer): Bool;
begin
  result := BOOL( SendMessage(hwnd, LVM_GETCOLUMNORDERARRAY, Longint(iCount), Longint(pi)) );
end;

function ListView_SetHotItem(hwnd: HWND; i: integer): integer;
begin
  result := integer( SendMessage(hwnd, LVM_SETHOTITEM, Longint(i), 0) );
end;

function ListView_GetHotItem(hwnd: HWND): integer;
begin
  result := integer( SendMessage(hwnd, LVM_GETHOTITEM, 0, 0) );
end;

function ListView_SetHotFont(hwnd: HWND; hcur: TCursor): TCursor;
begin
  result := HCURSOR( SendMessage(hwnd, LVM_SETHOTCURSOR, Longint(hcur), 0) );
end;

function ListView_GetHotFont(hwnd: HWND): TCursor;
begin
  result := HCURSOR( SendMessage(hwnd, LVM_GETHOTCURSOR, 0, 0) );
end;

function ListView_ApproximateViewRect(hwnd: HWND; iWidth, iHeight, iCount: integer): DWORD;
begin
  result := DWORD( SendMessage(hwnd, LVM_APPROXIMATEVIEWRECT, iCount, MAKELPARAM(iWidth, iHeight)) );
end;


end.
